import { createHash } from "crypto";
//import { murmur2 } from "murmurhash2";

/**
 * The hash formats allowed by Packwiz.
 */
export enum HashFormat {
  /**
   * SHA-256 hash format.
   */
  "sha256" = "sha256",
  /**
   * SHA-512 hash format.
   */
  "sha512" = "sha512",
  /**
   * SHA-1 hash format.
   */
  "sha1" = "sha1",
  /**
   * MD5 hash format.
   */
  "md5" = "md5",
  /**
   * Murmur2 hash format.
   * This is a proprietary hash format used by CurseForge.
   */
  "murmur2" = "murmur2",
}

/**
 * Checks if the provided hash format is allowed by Packwiz.
 * @param hashFormat - The hash format to check.
 * @returns The HashFormat enum value if valid, otherwise throws an error.
 */
export function isValidHashFormat(hashFormat: string): HashFormat {
  if (hashFormat in HashFormat) {
    return HashFormat[hashFormat as keyof typeof HashFormat];
  } else {
    throw new Error(`Invalid index file hash format: ${hashFormat}`);
  }
}

/**
 * Checks if the provided hash matches the computed hash of the provided data.
 * @param hash - The hash to check against.
 * @param hashFormat - The hashing format to use.
 * @param data - The data to hash and compare.
 * @returns true if the hashes match, false otherwise.
 */
export function doHashesMatch(
  hash: string,
  hashFormat: HashFormat,
  data: Buffer,
): boolean {
  let computedHash;
  if (hashFormat === HashFormat.murmur2) {
    console.warn(
      "WARNING: Murmur2 hash checking is not implemented yet! Skipping hash checking.",
    );
    return true;
    //computedHash = murmur2(data.toString(), 0).toString(16);
  } else {
    computedHash = createHash(hashFormat).update(data).digest("hex");
  }
  console.log(`Comparing ${hashFormat} hashes: ${hash} vs ${computedHash}`);
  return hash === computedHash;
}
